package com.qmth.exam.reserve.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.LambdaUpdateWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.qmth.boot.core.collection.PageResult;
import com.qmth.boot.core.concurrent.service.ConcurrentService;
import com.qmth.boot.core.exception.StatusException;
import com.qmth.boot.tools.excel.ExcelReader;
import com.qmth.boot.tools.excel.enums.ExcelType;
import com.qmth.boot.tools.excel.model.DataMap;
import com.qmth.exam.reserve.bean.applytask.CurrentApplyTaskVO;
import com.qmth.exam.reserve.bean.login.LoginUser;
import com.qmth.exam.reserve.bean.org.OrgInfo;
import com.qmth.exam.reserve.bean.room.ExamRoomReq;
import com.qmth.exam.reserve.bean.room.ExamRoomSaveReq;
import com.qmth.exam.reserve.bean.room.ExamRoomVO;
import com.qmth.exam.reserve.bean.timeperiod.TimePeriodExamSiteBean;
import com.qmth.exam.reserve.cache.CacheConstants;
import com.qmth.exam.reserve.cache.impl.ApplyTaskCacheService;
import com.qmth.exam.reserve.cache.impl.OrgCacheService;
import com.qmth.exam.reserve.dao.ExamRoomDao;
import com.qmth.exam.reserve.entity.ExamRoomEntity;
import com.qmth.exam.reserve.entity.ExamSiteEntity;
import com.qmth.exam.reserve.entity.TimePeriodEntity;
import com.qmth.exam.reserve.entity.TimePeriodExamRoomEntity;
import com.qmth.exam.reserve.service.*;
import com.qmth.exam.reserve.util.DateUtil;
import com.qmth.exam.reserve.util.PageUtil;
import org.apache.commons.collections4.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.redisson.api.RLock;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.transaction.interceptor.TransactionAspectSupport;
import org.springframework.transaction.support.TransactionSynchronization;
import org.springframework.transaction.support.TransactionSynchronizationManager;

import java.io.InputStream;
import java.util.*;
import java.util.function.Function;
import java.util.stream.Collectors;

@Service
public class ExamRoomServiceImpl extends ServiceImpl<ExamRoomDao, ExamRoomEntity> implements ExamRoomService {

    private static final Logger log = LoggerFactory.getLogger(ExamRoomServiceImpl.class);
    private static final String[] EXCEL_HEADER = new String[]{"考场号", "考场名称", "考场地址", "考场容量", "所属考点代码", "所属考点名称"};

    @Autowired
    private ExamSiteService examSiteService;

    @Autowired
    private CategoryService categoryService;

    @Autowired
    private ApplyTaskCacheService cacheService;

    @Autowired
    private TimePeriodService timePeriodService;

    @Autowired
    private TimePeriodExamRoomService timePeriodExamRoomService;

    @Autowired
    private ConcurrentService concurrentService;

    @Autowired
    private OrgCacheService orgCacheService;


    @Override
    public PageResult<ExamRoomVO> pageExamRoom(ExamRoomReq req) {
        IPage<ExamRoomVO> iPage = baseMapper.pageExamRoom(new Page<>(req.getPageNumber(), req.getPageSize()),
                req);
        return PageUtil.of(iPage);
    }

    @Transactional
    @Override
    public void saveExamRoom(LoginUser user, ExamRoomSaveReq req) {
        if (req.getExamSiteId() == null) {
            throw new StatusException("考点ID不能为空");
        }

        // 锁检查
        if (concurrentService.isLocked(CacheConstants.LOCK_AUTO_APPLY)) {
            log.warn("[考场保存]系统自动预约中，不允许修改考场！lockKey:{}", CacheConstants.LOCK_AUTO_APPLY);
            throw new StatusException("系统正在自动预约中，不允许修改考场");
        }

        //考点容量变更锁
        String examSiteLockKey = String.format(CacheConstants.LOCK_EXAM_SITE_CAPACITY, req.getExamSiteId());
        RLock examSiteLock = (RLock) concurrentService.getLock(examSiteLockKey);

        try {
            if (!examSiteLock.tryLock()) {
                log.warn("[考场保存] 获取锁失败，同一个教学点不允许同时操作一个考点的容量修改, lockKey:{}", examSiteLockKey);
                throw new StatusException("其他老师正在修改该考点的容量，不允许同时操作，请稍后再试！");
            } else {
                log.warn("[考场保存] 获取锁成功！lockKey:{}", examSiteLockKey);

                //校验
                checkExamRoom(req);

                ExamRoomEntity examRoomEntity = new ExamRoomEntity();
                BeanUtils.copyProperties(req, examRoomEntity);

                ExamRoomEntity beforeUpdateRoom = new ExamRoomEntity();
                //考点容量
                ExamSiteEntity examSite = examSiteService.getById(req.getExamSiteId());
                int oldCapacity = examSite.getCapacity();
                int newCapacity;
                if (req.getId() == null) {
                    examRoomEntity.setEnable(Boolean.TRUE);
                    save(examRoomEntity);
                } else {
                    beforeUpdateRoom = getById(req.getId());

                    if (req.getCapacity() != null && beforeUpdateRoom.getCapacity() != null && beforeUpdateRoom.getCapacity() > req.getCapacity()) {
                        checkCapacity(beforeUpdateRoom, beforeUpdateRoom.getCapacity() - req.getCapacity());
                    }

                    updateById(examRoomEntity);
                }
                //更新考点容量和教学点容量
                updateExamSiteAndTeachingCapacity(req.getExamSiteId());
                //考场更换考点
                if (req.getId() != null && !req.getExamSiteId().equals(beforeUpdateRoom.getExamSiteId())) {
                    updateExamSiteAndTeachingCapacity(beforeUpdateRoom.getExamSiteId());
                }

                // 获取更新后的容量
                examSite = examSiteService.getById(req.getExamSiteId());
                newCapacity = examSite.getCapacity();

                //刷新缓存
                doRefreshApplyCountCache(examRoomEntity.getId(), examRoomEntity.getExamSiteId(), oldCapacity, newCapacity);
            }
        } catch (Exception e) {
            log.error("[考场保存]保存失败，原因:{}", e.getMessage(), e);
            throw new StatusException(e.getMessage());
        } finally {
            try {
                if (examSiteLock.isLocked() && examSiteLock.isHeldByCurrentThread()) {
                    examSiteLock.unlock();
                    log.info("[考场保存] 解锁成功,lockKey:{}", examSiteLockKey);
                }
            } catch (Exception e) {
                log.warn(e.getMessage());
            }
        }
    }

    private void checkCapacity(ExamRoomEntity beforeUpdateRoom, Integer decreaseCapacity) {
        if (beforeUpdateRoom == null || beforeUpdateRoom.getExamSiteId() == null) {
            throw new StatusException("考场信息不完整");
        }

        OrgInfo org = orgCacheService.currentOrg();
        if (org == null) {
            throw new StatusException("当前机构不存在");
        }

        CurrentApplyTaskVO curApplyTask = cacheService.currentApplyTask(org.getOrgId());
        if (curApplyTask == null) {
            return;
        }

        List<TimePeriodExamSiteBean> timePeriodExamSiteBeans = timePeriodService.listTimePeriodByTask(curApplyTask.getTaskId());
        if (CollectionUtils.isEmpty(timePeriodExamSiteBeans)) {
            return;
        }

        //过滤已不可取消的时段
        timePeriodExamSiteBeans = filterTimePeriod(timePeriodExamSiteBeans, curApplyTask);
        if (CollectionUtils.isEmpty(timePeriodExamSiteBeans)) {
            return;
        }

        //获取更新前的时段容量
        Map<Long, Integer> oldCapacityMap = new HashMap<>();
        for (TimePeriodExamSiteBean timePeriod : timePeriodExamSiteBeans) {
            Long periodId = timePeriod.getTimePeriodId();
            int capacity = examSiteService.getExamSiteTimePeriodCapacity(beforeUpdateRoom.getExamSiteId(), periodId);
            oldCapacityMap.put(periodId, capacity);
        }

        Set<Long> periodIds = timePeriodExamSiteBeans.stream()
                .map(TimePeriodExamSiteBean::getTimePeriodId)
                .collect(Collectors.toSet());

        //考场对应的时段Map
        Map<Long, TimePeriodExamRoomEntity> timePeriodExamRoomMap = new HashMap<>();
        if (!periodIds.isEmpty()) {
            List<TimePeriodExamRoomEntity> timePeriodExamRoomList = timePeriodExamRoomService.listByExamRoomAndPeriodIds(beforeUpdateRoom.getId(), periodIds);
            timePeriodExamRoomMap = timePeriodExamRoomList.stream()
                    .collect(Collectors.toMap(
                            TimePeriodExamRoomEntity::getTimePeriodId,
                            Function.identity(),
                            (existing, replacement) -> existing
                    ));
        }

        for (TimePeriodExamSiteBean bean : timePeriodExamSiteBeans) {
            //判断考场+时段的是否已经关闭
            TimePeriodExamRoomEntity timePeriodExamRoom = timePeriodExamRoomMap.get(bean.getTimePeriodId());
            //已关闭的考场时段不处理
            if (timePeriodExamRoom != null && !timePeriodExamRoom.getEnable()) {
                continue;
            }

            Long periodId = bean.getTimePeriodId();
            //剩余的容量
            int availableCount = cacheService.getApplyAvailableCount(beforeUpdateRoom.getExamSiteId(), periodId);
            //修改前的容量
            int oldCount = oldCapacityMap.getOrDefault(periodId, 0);
            if (oldCount <= 0) {
                continue;
            }
            log.warn("[考场容量检测]periodId:{}, availableCount:{}, oldCount:{}", periodId, availableCount, oldCount);

            //已预约的容量
            int haveApplyCount = oldCount - availableCount;
            if (haveApplyCount <= 0) {
                continue;
            }
            //即将更新之后的剩余容量
            int remainCount = oldCount - decreaseCapacity;
            if (haveApplyCount > remainCount) {
                TimePeriodEntity timePeriod = timePeriodService.getById(periodId);
                if (timePeriod == null) {
                    continue;
                }
                String dateStr = DateUtil.getShortDateByLongTime(timePeriod.getStartTime());
                String timeStr = DateUtil.getStartToEndTime(timePeriod.getStartTime(), timePeriod.getEndTime());
                String errorMessage = String.format("时段：%s %s，已预约%d人，当前已预约人数超出剩余容量%d人，无法缩小考场容量。", dateStr, timeStr,
                        haveApplyCount, remainCount);
                log.error(errorMessage);
                throw new StatusException(errorMessage);
            }
        }
    }

    private List<TimePeriodExamSiteBean> filterTimePeriod(List<TimePeriodExamSiteBean> timePeriodExamRoomList, CurrentApplyTaskVO curApplyTask) {
        List<TimePeriodExamSiteBean> resultList = new ArrayList<>();
        if (CollectionUtils.isNotEmpty(timePeriodExamRoomList)) {
            Long longToday = DateUtil.getLongTimeByDate(DateUtil.formatShortSplitDateString(new Date()) + " 00:00:00");
            Date today = new Date(longToday);
            Date otherDay = DateUtil.addValues(today, Calendar.DAY_OF_MONTH, curApplyTask.getAllowApplyCancelDays());
            long longOtherDay = DateUtil.getLongTimeByDate(DateUtil.formatShortSplitDateString(otherDay) + " 23:59:59");
            // 当前时间
            long now = System.currentTimeMillis();

            for (TimePeriodExamSiteBean bean : timePeriodExamRoomList) {
                if (isTimeEditable(now, bean.getEndTime(), longOtherDay, bean.getStartTime())) {
                    resultList.add(bean);
                }
            }
        }
        return resultList;
    }

    private boolean isTimeEditable(long now, long endTime, long longOtherDay, long startTime) {
        return !(endTime < now || startTime < longOtherDay);
    }


    @Transactional
    @Override
    public void enable(Long id, Boolean enable) {
        if (concurrentService.isLocked(CacheConstants.LOCK_AUTO_APPLY)) {
            log.warn("[考场禁用/启用]系统自动预约中，不允许启用/禁用考场！lockKey:{}", CacheConstants.LOCK_AUTO_APPLY);
            throw new StatusException("系统正在自动预约中，不允许启用/禁用考场");
        }
        ExamRoomEntity examRoom = getById(id);

        //考点容量变更锁
        String examSiteLockKey = String.format(CacheConstants.LOCK_EXAM_SITE_CAPACITY, examRoom.getExamSiteId());
        RLock examSiteLock = (RLock) concurrentService.getLock(examSiteLockKey);

        try {
            if (!examSiteLock.tryLock()) {
                log.warn("[考场禁用/启用] 获取锁失败，同一个教学点不允许同时操作一个考点的容量修改, lockKey:{}", examSiteLockKey);
                throw new StatusException("其他老师正在修改该考点的容量，不允许同时操作，请稍后再试！");
            } else {
                log.warn("[考场禁用/启用] 获取锁成功！lockKey:{}", examSiteLockKey);

                //考点容量
                ExamSiteEntity examSite = examSiteService.getById(examRoom.getExamSiteId());
                int oldCapacity = examSite.getCapacity();

                //禁用的时候，判断容量
                if (!enable) {
                    checkCapacity(examRoom, examRoom.getCapacity());
                }

                LambdaUpdateWrapper<ExamRoomEntity> wrapper = new LambdaUpdateWrapper<>();
                wrapper.set(ExamRoomEntity::getEnable, enable);
                wrapper.eq(ExamRoomEntity::getId, id);
                update(null, wrapper);

                updateExamSiteAndTeachingCapacity(examRoom.getExamSiteId());

                //更新之后考点的容量
                examSite = examSiteService.getById(examRoom.getExamSiteId());

                //刷新缓存
                doRefreshApplyCountCache(id, examRoom.getExamSiteId(), oldCapacity, examSite.getCapacity());
            }
        } catch (Exception e) {
            log.error("[考场禁用/启用]保存失败，原因:{}", e.getMessage(), e);
            throw new StatusException(e.getMessage());
        } finally {
            try {
                if (examSiteLock.isLocked() && examSiteLock.isHeldByCurrentThread()) {
                    examSiteLock.unlock();
                    log.info("[考场禁用/启用] 解锁成功,lockKey:{}", examSiteLockKey);
                }
            } catch (Exception e) {
                log.warn(e.getMessage());
            }
        }


    }

    @Transactional
    @Override
    public List<Map<String, Object>> importExamRoom(LoginUser user, InputStream inputStream) {
        if (concurrentService.isLocked(CacheConstants.LOCK_AUTO_APPLY)) {
            log.warn("[考场导入]系统自动预约中，不允许导入考场！lockKey:{}", CacheConstants.LOCK_AUTO_APPLY);
            throw new StatusException("系统正在自动预约中，不允许导入考场");
        }

        List<DataMap> lineList;
        ExcelReader reader = ExcelReader.create(ExcelType.XLSX, inputStream, 0);
        try {
            lineList = reader.getDataMapList();
        } catch (Exception e) {
            throw new StatusException("Excel 解析失败");
        }
        if (com.baomidou.mybatisplus.core.toolkit.CollectionUtils.isEmpty(lineList)) {
            throw new StatusException("Excel无内容");
        }
        List<Map<String, Object>> failRecords = new ArrayList<>();
        List<ExamRoomEntity> examRoomList = new ArrayList<>();
        Long teachingId = user.getCategoryId();
        for (int i = 0; i < lineList.size(); i++) {
            DataMap line = lineList.get(i);
            ExamRoomEntity room = new ExamRoomEntity();
            StringBuilder msg = new StringBuilder();
            String code = trimAndNullIfBlank(line.get(EXCEL_HEADER[0]));
            if (StringUtils.isBlank(code)) {
                msg.append(" 考场号不能为空");
            } else if(code.length() > 20) {
                msg.append(" 考场号不能超过20个字符");
            } else {
                room.setCode(code);
            }
            String name = trimAndNullIfBlank(line.get(EXCEL_HEADER[1]));
            if (StringUtils.isBlank(name)) {
                msg.append(" 考场名称不能为空");
            } else if (name.length() > 50) {
                msg.append(" 考场名称不能超过50个字符");
            } else {
                room.setName(name);
            }
            String address = trimAndNullIfBlank(line.get(EXCEL_HEADER[2]));
            if (StringUtils.isBlank(address)) {
                msg.append(" 考场地址不能为空");
            } else if (address.length() > 100) {
                msg.append(" 考场地址不能超过100个字符");
            } else {
                room.setAddress(address);
            }
            String capacity = trimAndNullIfBlank(line.get(EXCEL_HEADER[3]));
            if (StringUtils.isBlank(capacity) || !capacity.matches("\\d+") || Integer.parseInt(capacity) <= 0 || Integer.parseInt(capacity) > 99999) {
                msg.append(" 考场容量必须为大于0且小于100000的整数");
            } else {
                room.setCapacity(Integer.parseInt(capacity));
            }
            String examSiteCode = trimAndNullIfBlank(line.get(EXCEL_HEADER[4]));
            if (StringUtils.isBlank(examSiteCode)) {
                msg.append(" 所属考点代码不能为空");
            }
            String examSiteName = trimAndNullIfBlank(line.get(EXCEL_HEADER[5]));
            if (StringUtils.isBlank(examSiteName)) {
                msg.append(" 所属考点名称不能为空");
            }
            List<ExamSiteEntity> examSiteList = listExamSite(examSiteCode, examSiteName, teachingId);
            if (examSiteList.isEmpty()) {
                msg.append(" 找不到所属考点");
            }
            if (examSiteList.size() > 1) {
                msg.append(" 找到的所属考点大于1个");
            }
            if (examSiteList.size() == 1) {
//                ExamRoomEntity examRoom = getExamRoom(examSiteList.get(0).getId(), code);
//                if (examRoom != null) {
//                    msg.append(" 当前考点下已经存在考场代码为：").append(code).append("的考场");
//                } else {
                    room.setExamSiteId(examSiteList.get(0).getId());
                    room.setEnable(Boolean.TRUE);
//                }
            }

            if (msg.length() > 0) {
                failRecords.add(newError(i + 1, msg.toString()));
            } else {
                examRoomList.add(room);
            }
        }

        if (!failRecords.isEmpty())
            return failRecords;

        for (int i = 0; i < examRoomList.size(); i++) {
            ExamRoomEntity examRoomEntity = examRoomList.get(i);
            try {
                saveRoom(examRoomEntity);
            } catch (Exception e) {
                failRecords.add(newError(i + 1, " " + e.getMessage()));
                log.error("导入异常", e);
            }
        }

        if (CollectionUtils.isNotEmpty(failRecords)) {
            TransactionAspectSupport.currentTransactionStatus().setRollbackOnly();
            return failRecords;
        }

        return failRecords;
    }

    @Override
    public List<ExamRoomEntity> listExamRoom(Long examSiteId) {
        LambdaQueryWrapper<ExamRoomEntity> wrapper = new LambdaQueryWrapper<>();
        wrapper.eq(ExamRoomEntity::getExamSiteId, examSiteId);
        wrapper.eq(ExamRoomEntity::getEnable, Boolean.TRUE);
        wrapper.orderByAsc(ExamRoomEntity::getCode);
        return list(wrapper);
    }

    @Override
    public List<ExamRoomEntity> listExamRoomByExamSiteId(Long examSiteId) {
        return baseMapper.listExamRoomByExamSiteId(examSiteId, Boolean.TRUE);
    }

    private void saveRoom(ExamRoomEntity room) {
        //考点容量变更锁
        String examSiteLockKey = String.format(CacheConstants.LOCK_EXAM_SITE_CAPACITY, room.getExamSiteId());
        RLock examSiteLock = (RLock) concurrentService.getLock(examSiteLockKey);

        try {
            if (!examSiteLock.tryLock()) {
                log.warn("[考场导入] 获取锁失败，同一个教学点不允许同时操作一个考点的容量修改, lockKey:{}", examSiteLockKey);
                throw new StatusException("其他老师正在修改该考点的容量，不允许同时操作，请稍后再试！");
            } else {
                log.warn("[考场导入] 获取锁成功! lockKey:{}", examSiteLockKey);

                //保存之前的考点容量
                ExamSiteEntity examSite = examSiteService.getById(room.getExamSiteId());
                if (examSite == null) {
                    throw new StatusException("找不到考点");
                }
                int oldCapacity = examSite.getCapacity();
                Long roomId;

                ExamRoomEntity examRoom = getExamRoom(room.getExamSiteId(), room.getCode());
                if (examRoom != null) {

                    // 校验容量
                    if (examRoom.getCapacity() != null && room.getCapacity() != null && examRoom.getCapacity() > room.getCapacity()) {
                        checkCapacity(examRoom, examRoom.getCapacity() - room.getCapacity());
                    }

                    roomId = examRoom.getId();
                    examRoom.setCapacity(room.getCapacity());
                    LambdaUpdateWrapper<ExamRoomEntity> wrapper = new LambdaUpdateWrapper<>();
                    wrapper.set(ExamRoomEntity::getName, room.getName());
                    wrapper.set(ExamRoomEntity::getAddress, room.getAddress());
                    wrapper.set(ExamRoomEntity::getCapacity, room.getCapacity());
                    wrapper.set(ExamRoomEntity::getEnable, Boolean.TRUE);
                    wrapper.eq(ExamRoomEntity::getId, examRoom.getId());
                    update(null, wrapper);
                } else {
                    save(room);
                    roomId = room.getId();
                }
                //更新考点、教学点容量
                updateExamSiteAndTeachingCapacity(room.getExamSiteId());
                //更新之后的容量
                examSite = examSiteService.getById(room.getExamSiteId());

                //刷新缓存
                //refreshApplyCountCacheAfterCommit(roomId, room.getExamSiteId(), oldCapacity,  examSite.getCapacity());
                doRefreshApplyCountCache(roomId, room.getExamSiteId(), oldCapacity, examSite.getCapacity());
            }
        } catch (Exception e) {
            log.error("[考场导入]保存失败，原因:{}", e.getMessage(), e);
            throw new StatusException(e.getMessage());
        } finally {
            try {
                if (examSiteLock.isLocked() && examSiteLock.isHeldByCurrentThread()) {
                    examSiteLock.unlock();
                    log.info("[考场导入] 解锁成功,lockKey:{}", examSiteLockKey);
                }
            } catch (Exception e) {
                log.warn(e.getMessage());
            }
        }

    }

    private void refreshApplyCountCacheAfterCommit(Long examRoomId, Long examSiteId, int oldCapacity, int newCapacity) {
        if (TransactionSynchronizationManager.isActualTransactionActive()) {
            TransactionSynchronizationManager.registerSynchronization(new TransactionSynchronization() {

                @Override
                public void afterCommit() {
                    doRefreshApplyCountCache(examRoomId, examSiteId, oldCapacity, newCapacity);
                }
            });
        } else {
            doRefreshApplyCountCache(examRoomId, examSiteId, oldCapacity, newCapacity);
        }
    }

    private void doRefreshApplyCountCache(Long examRoomId, Long examSiteId, int oldCapacity, int newCapacity) {
        List<TimePeriodEntity> timePeriods = timePeriodService.list(
                new LambdaQueryWrapper<TimePeriodEntity>().select(TimePeriodEntity::getId));

        List<TimePeriodExamRoomEntity> timePeriodExamRooms = timePeriodExamRoomService.listExamRoom(examRoomId);

        Map<Long, TimePeriodExamRoomEntity> timePeriodExamRoomMap = Optional.ofNullable(timePeriodExamRooms)
                .orElse(Collections.emptyList())
                .stream()
                .collect(Collectors.toMap(
                        TimePeriodExamRoomEntity::getTimePeriodId,
                        Function.identity(),
                        (existing, replacement) -> existing
                ));

        for (TimePeriodEntity timePeriod : timePeriods) {
            TimePeriodExamRoomEntity timePeriodExamRoomEntity = timePeriodExamRoomMap.get(timePeriod.getId());
            if (timePeriodExamRoomEntity == null || timePeriodExamRoomEntity.getEnable()) {
                cacheService.refreshApplyAvailableCountCache(examSiteId, timePeriod.getId(), oldCapacity, newCapacity);
            }
        }
    }


    private List<ExamSiteEntity> listExamSite(String examSiteCode, String examSiteName, Long categoryId) {
        LambdaQueryWrapper<ExamSiteEntity> wrapper = new LambdaQueryWrapper<>();
        wrapper.eq(ExamSiteEntity::getCode, examSiteCode);
        wrapper.eq(ExamSiteEntity::getName, examSiteName);
        wrapper.eq(categoryId != null, ExamSiteEntity::getCategoryId, categoryId);
        return examSiteService.list(wrapper);
    }

    private void updateExamSiteAndTeachingCapacity(Long examSiteId) {
        // 更新考点容量
        examSiteService.updateExamSiteCapacity(examSiteId);
        ExamSiteEntity newExamSite = examSiteService.getById(examSiteId);
        //更新教学点容量
        categoryService.updateTeachingCapacity(newExamSite.getCategoryId());
    }

    private void checkExamRoom(ExamRoomSaveReq req) {
        if (StringUtils.isEmpty(req.getCode())) {
            throw new StatusException("考场代码不能为空");
        }
        if (StringUtils.isEmpty(req.getName())) {
            throw new StatusException("考场名称不能为空");
        }
        if (StringUtils.isEmpty(req.getAddress())) {
            throw new StatusException("考场地址不能为空");
        }
        if (req.getCapacity() == null || req.getCapacity() <= 0) {
            throw new StatusException("考场容量必须大于0");
        }
        if (req.getExamSiteId() == null) {
            throw new StatusException("请选择考点");
        }
        checkExamRoomExist(req, req.getId() != null);
    }

    private void checkExamRoomExist(ExamRoomSaveReq req, boolean idFlag) {
        LambdaQueryWrapper<ExamRoomEntity> wrapper = new LambdaQueryWrapper<>();
        wrapper.eq(ExamRoomEntity::getExamSiteId, req.getExamSiteId());
        wrapper.eq(ExamRoomEntity::getCode, req.getCode());
        wrapper.notIn(idFlag, ExamRoomEntity::getId, req.getId());
        ExamRoomEntity examRoomEntity = baseMapper.selectOne(wrapper);
        if (examRoomEntity != null) {
            ExamSiteEntity site = examSiteService.getById(req.getExamSiteId());
            throw new StatusException(site.getName() + "下已存在代码为:" + req.getCode() + "的考场");
        }
    }

    private ExamRoomEntity getExamRoom(Long examSiteId, String roomCode) {
        LambdaQueryWrapper<ExamRoomEntity> wrapper = new LambdaQueryWrapper<>();
        wrapper.eq(ExamRoomEntity::getExamSiteId, examSiteId);
        wrapper.eq(ExamRoomEntity::getCode, roomCode);
        return baseMapper.selectOne(wrapper);
    }

    private String trimAndNullIfBlank(String s) {
        if (StringUtils.isBlank(s)) {
            return null;
        }
        return s.trim();
    }

    private Map<String, Object> newError(int lineNum, String msg) {
        Map<String, Object> map = new HashMap<>();
        map.put("lineNum", lineNum);
        map.put("msg", msg);
        return map;
    }
}
